#include <mpi.h>
#include <mpi-ext.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

int main(int argc, char *argv[])
{
    int rank, size, rc, len, g_size, i;
    int *ranks1, *ranks2;
    char errstr[MPI_MAX_ERROR_STRING];
    MPI_Group group, cgroup;

    MPI_Init(NULL, NULL);
    MPI_Comm_set_errhandler(MPI_COMM_WORLD,
                            MPI_ERRORS_RETURN);

    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &size);
    if( rank == (size-1) ) raise(SIGKILL);

    rc = MPI_Barrier(MPI_COMM_WORLD);
    MPI_Error_string( rc, errstr, &len );
    if( MPI_ERR_PROC_FAILED == rc ) {
        OMPI_Comm_failure_ack(MPI_COMM_WORLD);
        OMPI_Comm_failure_get_acked(MPI_COMM_WORLD, &group);
        MPI_Comm_group(MPI_COMM_WORLD, &cgroup);
        MPI_Group_size(group, &g_size);
        ranks1 = (int*)malloc(g_size * sizeof(int));
        ranks2 = (int*)malloc(g_size * sizeof(int));
        for(i = 0; i < g_size; ranks1[i] = i, i++ );
        MPI_Group_translate_ranks(group, g_size, ranks1,
                                  cgroup, ranks2);
        printf("Rank %d / %d (error %s) [%d dead: ",
               rank, size, errstr, g_size);
        for(i = 0; i < g_size; ranks1[i] = i, i++ )
            printf("%d ", ranks2[i]);
        printf("]\n");
    } else {
        printf("Rank %d / %d (error NONE)\n",
               rank, size);
    }

    MPI_Finalize();
}
